/*
Copyright Caroline et Serge COUDÉ, (06/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/
package net.sergecoude.TbiCetidem;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.net.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import net.sergecoude.TbiGeneral.*;

public class Element extends bouton implements Cloneable { 

  private final static long serialVersionUID = 1L;
  
	protected String _nom;
	protected URL _urlNom;
	protected boolean _coche = false;
	protected ByteArrayInputStream _rawAudio = null;
	protected String _id = "";
	protected Color _couleurCoche = new Color(0,180,0);
	protected boolean _identique = false; 

  @Override
  public Object clone() throws CloneNotSupportedException {
    Element copieElement = (Element)super.clone();
    return copieElement;
  }  

  public Element(String nom, String id, URL ressourceImage, URL ressourceNom) throws Exception { 
    super(ressourceImage);
		_nom = nom;
    _urlNom = ressourceNom;
    _id = id;
	}
			
  public Element(String nom, URL ressourceImage, URL ressourceNom) throws Exception {
      this(nom, nom, ressourceImage, ressourceNom); 
	}
	
	 public Element(String nom,URL ressourceNom) throws Exception { 
	  super(nom);
		_nom = nom;
    _urlNom = ressourceNom;
	}
	
	public Element(ItemEleve iEleve) throws Exception { 
		_nom = iEleve.getEleve().getNomUsuel();
    _urlNom = null;
    _rawAudio = iEleve.getAudio();
    _id = iEleve.getEleve().getId();
	}
	
  public static BufferedImage toBufferedImage(ItemEleve iEleve, int w, int h) {
    Image image = iEleve.getImage();
    boolean hasAlpha = true;
    if (image != null) {
      image = new ImageIcon(image).getImage();
      hasAlpha = hasAlpha(image);
    }
    BufferedImage bimage = null;
    GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
    try {
      int transparency = Transparency.OPAQUE;
      if (hasAlpha) 
        transparency = Transparency.BITMASK;
      GraphicsDevice gs = ge.getDefaultScreenDevice();
      GraphicsConfiguration gc = gs.getDefaultConfiguration();
      bimage = gc.createCompatibleImage( w, h, transparency);
    } catch (HeadlessException e) {
    }
    if (bimage == null) {
      int type = BufferedImage.TYPE_INT_RGB;
      if (hasAlpha) 
        type = BufferedImage.TYPE_INT_ARGB;
      bimage = new BufferedImage(w, h, type);
    }
    Graphics2D g = bimage.createGraphics();
    if (image != null)
      g.drawImage(image, 0, 0, w, w, null);
    else { // si l'image n'existe pas, alors une croix noire
      Stroke oldStroke = g.getStroke();
      g.setStroke(new BasicStroke(5));
      g.setColor(Color.WHITE);
      g.fillRect(0,0,w,w);
      g.setColor(Color.BLACK);
      g.drawLine(0,0,w,w);
      g.drawLine(w,0,0,w);
      g.setStroke(oldStroke);
    }
    g.setColor(Color.WHITE);
    g.fillRect(0,w,w,h);
    g.setFont(new Font("SansSerif",Font.BOLD,12));
    g.setColor(Color.BLACK);
    Dimension fontSize = foundFontSize(g, iEleve.getEleve().getNomUsuel(), w, h-w);
    g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
    g.drawString(iEleve.getEleve().getNomUsuel(),(w-fontSize.width)/2,h-g.getFontMetrics().getDescent());
    g.dispose();
    return bimage;
  }
  
  public static Dimension foundFontSize(Graphics2D g, String chaine, int largeur, int hauteur) {
    int l = 0, h = 0, s = 0;
    while(l <= largeur && h <= hauteur) {
      s++;
      g.setFont(g.getFont().deriveFont((float)s));
      FontMetrics fontMetrics = g.getFontMetrics();
      l = fontMetrics.stringWidth(chaine);
      h = fontMetrics.getAscent()+fontMetrics.getDescent();
    }
    g.setFont(g.getFont().deriveFont((float)(s-1)));
    FontMetrics fontMetrics = g.getFontMetrics();
    l = fontMetrics.stringWidth(chaine);
    h = fontMetrics.getAscent()+fontMetrics.getDescent();
    return new Dimension(l, h);
  } 
  
  public static boolean hasAlpha(Image image) {
    if (image instanceof BufferedImage) {
        BufferedImage bimage = (BufferedImage)image;
        return bimage.getColorModel().hasAlpha();
    }
    PixelGrabber pg = new PixelGrabber(image, 0, 0, 1, 1, false);
    try {
        pg.grabPixels();
    } catch (InterruptedException e) {
    }
    ColorModel cm = pg.getColorModel();
    return cm.hasAlpha();  
  }

  public String toString() {
    return _nom;
  }

  public String getNom() {
    return _nom;
  }
  
  public URL getURLNom() {
    return _urlNom;
  } 
  
  public void setId(String id) {
    _id = id;
  }
  
  public String getId() {
    return _id;
  }  
  
  public ByteArrayInputStream getAudio() {
    return _rawAudio;
  }
  
  public boolean identique() {
    return _identique;
  }
  
  public void setIdentique(boolean oui) {
    _identique = oui;
  }
  
  public void setCoche(boolean oui) {
    _coche = oui;
    repaint();
  }

  public boolean estCoche() {
    return _coche;
  }
  
  public void setRessourceImage(URL ressourceImage) throws Exception { 
	  setImage(ressourceImage);
	}
  
  public void setRessourceAudio(URL ressourceAudio) { 
    _urlNom = ressourceAudio;
  }   

  private BufferedImage pivote(BufferedImage image, int angle) {
      int wNew = image.getWidth(null);
      int hNew = image.getHeight(null);
      if (angle == 90) {
        BufferedImage image90 = new BufferedImage(hNew, wNew, BufferedImage.TYPE_INT_ARGB);
        AffineTransform a90 = AffineTransform.getRotateInstance(Math.toRadians(90), hNew/2, hNew/2);
        AffineTransformOp op90 = new AffineTransformOp(a90, AffineTransformOp.TYPE_BILINEAR);
        op90.filter(image,image90);
        return image90;
      }
      if (angle == 180) {
        BufferedImage image180 = new BufferedImage(wNew, hNew, BufferedImage.TYPE_INT_ARGB);
        AffineTransform a180 = AffineTransform.getRotateInstance(Math.toRadians(180), wNew/2, hNew/2);
        AffineTransformOp op180 = new AffineTransformOp(a180, AffineTransformOp.TYPE_BILINEAR);
        op180.filter(image,image180);
        return image180;
      }
      if (angle == 270) {      
        BufferedImage image270 = new BufferedImage(hNew, wNew, BufferedImage.TYPE_INT_ARGB);
        AffineTransform a270 = AffineTransform.getRotateInstance(Math.toRadians(270), wNew/2, wNew/2);
        AffineTransformOp op270 = new AffineTransformOp(a270,  AffineTransformOp.TYPE_BILINEAR);
        op270.filter(image,image270);
        return image270;
      }
      return image;
  }

  public void pivote(int angle) {
    if (image_fond != null) {
      image_fond = pivote(image_fond, angle);
    }
    if (image_fond_enfoncee != null) {
      image_fond_enfoncee = pivote(image_fond_enfoncee, angle);
    }
    _nom += angle;
    repaint();
  }
  
  @Override
	public void paint(Graphics g) {
	  super.paint(g);
		if (_coche) {
		  Dimension dim = getSize();
		  Graphics2D g2d = (Graphics2D)g;
		  g2d.setColor(_couleurCoche);
      float dx = ((float)dim.width*0.075f);
      g2d.setStroke(new BasicStroke(dx));
      Shape circle = new Ellipse2D.Float(dx/2, dx/2, (float)dim.width-dx, (float)dim.height-dx);
      g2d.draw(circle);      
    }
	}  
}


