/*
Copyright Caroline et Serge COUDÉ, (06/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/

package net.sergecoude.TbiCetidem;

import java.util.*;
import java.io.*;
import net.n3.nanoxml.*;
import net.sergecoude.TbiGeneral.*;

public class EvaluationCetidem extends Evaluation  {

  private final static long serialVersionUID = 1L;
  
  private int _nbImages;
  private String _quantiteIdentique;
  private boolean _avecRotation;
  private String _visibiliteModele;

  public EvaluationCetidem(TbiCetidem app) {
    super(app);
  }
  
  public EvaluationCetidem(TbiCetidem app, String chemin) throws Exception {
    super(app, chemin);      
  }

  public EvaluationCetidem(TbiApp app) {
    super(app);
  }
  
  public EvaluationCetidem(TbiApp app, String chemin) throws Exception {
    super(app, chemin);      
  }
  
  public void chargeEvaluation(IXMLElement xml) throws Exception {
    super.chargeEvaluation(xml);
		IXMLElement xNbImages = xml.getFirstChildNamed(TbiCetidem._V_NbImagesProposees); 
		if (xNbImages != null) {
		  setNbImages(Integer.parseInt(xNbImages.getContent()));
    } else 
      throwNewException("Le nombre d'images de l'évaluation n'est pas indiqué... [" + _chemin + "]");
		IXMLElement xQuantiteIdentique = xml.getFirstChildNamed(TbiCetidem._V_QuantiteIdentique);
		if (xQuantiteIdentique != null) {
		  setQuantiteIdentique(xQuantiteIdentique.getContent());
    } else 
      throwNewException("La quantité des images identiques de l'évaluation n'est pas indiquée... [" + _chemin + "]");
		IXMLElement xRotation = xml.getFirstChildNamed(TbiCetidem._V_Rotation);
		if (xRotation != null) {
		  setRotation(xRotation.getContent().equals("1"));
    } else 
      throwNewException("L'usage possible de la rotation de l'évaluation n'est pas indiqué... [" + _chemin + "]");
		IXMLElement xVisibiliteModele = xml.getFirstChildNamed(TbiCetidem._V_VisibiliteModele);
		if (xVisibiliteModele != null) {
		  setVisibiliteModele(xVisibiliteModele.getContent());
    } else
      throwNewException("La visibilité du modèle de l'évaluation n'est pas indiquée... [" + _chemin + "]");
  }
  
  @Override
  public Epreuve getNewEpreuve(Classe classe, IXMLElement xmlEpreuve) throws Exception {
    return new EpreuveCetidem(_app, classe, xmlEpreuve);
  }  
  
  public void setNbImages(int nbImages) {
    _nbImages = nbImages;
  }
  
  public int getNbImages() {
    return _nbImages;
  }

  public void setQuantiteIdentique(String quantIdent) {
    _quantiteIdentique = quantIdent;
  }
  
  public String getQuantiteIdentique() {
    return _quantiteIdentique;
  }
  
  public void setRotation(boolean rotation) {
    _avecRotation = rotation;
  }
  
  public boolean getRotation() {
    return _avecRotation;
  }

  public void setVisibiliteModele(String visibilite) {
    _visibiliteModele = visibilite;
  }
  
  public String getVisibiliteModele() {
    return _visibiliteModele;
  }
  
  public String toXmlDebut() {
    String xml = "<" + TbiCetidem._V_NbImagesProposees + ">" + getNbImages() + "</" + TbiCetidem._V_NbImagesProposees + ">";
    xml += "<" + TbiCetidem._V_QuantiteIdentique + ">" + getQuantiteIdentique() + "</" + TbiCetidem._V_QuantiteIdentique + ">";
    xml += "<" + TbiCetidem._V_Rotation + ">" + (getRotation() ? "1" : "0") + "</" + TbiCetidem._V_Rotation + ">";
    xml += "<" + TbiCetidem._V_VisibiliteModele + ">" + getVisibiliteModele() + "</" + TbiCetidem._V_VisibiliteModele + ">";
    return super.toXmlDebut() + xml;
  }

  @Override
  public String getCodeNiveauDifficulte() {
    StringTokenizer st = new StringTokenizer(getCle(), "-");
    // récupère les 3 dernières valeurs
    String c1 = ""; // // visibilite du modèle
    String c2 = ""; // // le nombre d'images proposées
    String c3 = ""; // le niveau d'images identiques proposées
    String c4 = "";  // rotation proposée : 1 oui, 0 non
    while(st.hasMoreTokens()) {
      c1 = c2;
      c2 = c3;
      c3 = c4;
      c4 = st.nextToken();
    }
    return c1+"-"+c2+"-"+c3+"-"+c4;
  }
   
}
