/*
Copyright Caroline et Serge COUDÉ, (05/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/
package net.sergecoude.TbiCetidem;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import javax.swing.event.*;
import net.sergecoude.TbiGeneral.*;

public class PanelSelectTheme extends Panneau implements ListSelectionListener {

  private final static long serialVersionUID = 1L;

  private JList _listeThemes;
  private JScrollPane _listeScroller;
  private ThemeListRenderer _themeListRenderer;
  private ResizeLabelFont _lThemeEtChemin;
	
  public PanelSelectTheme(TbiCetidem app) {
    super(app);
    
		_listeThemes = new JList(((TbiCetidem)_app).getLesThemes());
		_listeThemes.setFont(_police);
		_listeThemes.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    _listeThemes.setLayoutOrientation(JList.VERTICAL);
    _listeThemes.getSelectionModel().addListSelectionListener(this);
    _themeListRenderer = new ThemeListRenderer(_hauteurPolice);
    _themeListRenderer.setRatio(1f);
    _listeThemes.setCellRenderer(_themeListRenderer);
		_listeScroller = new JScrollPane(_listeThemes, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    _listeScroller.setOpaque(false);
    _listeScroller.setMinimumSize(new Dimension(700, 350));
    
    try {
      _bPrecedent = new bouton(_app.getRessourceByConf("boutonprecedent"));
      _bPrecedent.addActionListener(this);
    } catch (Exception e) {
      _app.log(e);
    }

    JPanel pMain = new JPanel();
    pMain.setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    pMain.setOpaque(false);
    setLayout(new BorderLayout());
    File repThemes = new File(_app.getConfiguration("chemindesthemes"));
    _lThemeEtChemin = new ResizeLabelFont("");
    _lThemeEtChemin.setFont(_police);
    _listeScroller.setAlignmentX(CENTER_ALIGNMENT);
    c.fill = GridBagConstraints.NONE;
    c.ipady = 40;      
    c.weightx = 0.0;
    c.gridwidth = 1;
    c.gridheight = 1;
    c.gridx = 0;
    c.gridy = 0;
    pMain.add(_lThemeEtChemin, c);
    c.gridwidth = 1;
    c.gridheight = 2;
    c.gridx = 0;
    c.gridy = 1;
    pMain.add(_listeScroller, c);
    c.gridwidth = 1;
    c.gridheight = 1;
    c.gridx = 0;
    c.gridy = 3;
    JPanel pPrecedent = new JPanel(new FlowLayout(FlowLayout.CENTER));
    pPrecedent.setOpaque(false);
    pPrecedent.add(_bPrecedent);
    pMain.add(pPrecedent, c);    
    add(pMain);
    _bPrecedent.setVisible(false);
  }
  
  public void setEnabled(boolean enabled) {
    _listeThemes.setEnabled(enabled);
  }
  
  public void init() {
    super.init();
    File repThemes = new File(_app.getConfiguration("chemindesthemes"));
    _lThemeEtChemin.setText("Choix du thème - " + repThemes.getAbsolutePath());
    _listeThemes.clearSelection();
    if (_app.getUtilisationDesClasses()) { // on va stoper l'évaluation précédente
      if (_app.getEvaluation() != null)
        _app.getEvaluation().stop();
    }
  }
  
  public void agenceLesComposants() {
    Dimension d = getSize();
    int w = 3*d.width/4;
    int h = d.height/2;
    Dimension dListeScroller = new Dimension(w, h);
    _listeScroller.setMinimumSize(dListeScroller);
    _listeScroller.setMaximumSize(dListeScroller);
    _listeScroller.setPreferredSize(dListeScroller);
    // maj de la police
    float surf = 1024 * 768;
    float surfTbi = _app.getLargeurTbi() * _app.getHauteurTbi();
    float ratio = (float)Math.sqrt(surfTbi / surf);
    if (ratio > 1)
      ratio = 1;
    _themeListRenderer.setRatio(ratio);
    int hauteurItem = (int)(_hauteurPolice*2*ratio);
    _listeThemes.setFixedCellHeight(hauteurItem);
    _themeListRenderer.setHauteur(hauteurItem);
    _listeThemes.revalidate();
    _listeScroller.revalidate();
    Dimension dLabels = new Dimension(w,hauteurItem);
    _lThemeEtChemin.setMinimumSize(dLabels);
    _lThemeEtChemin.setMaximumSize(dLabels);
    _bPrecedent.setRatio(ratio);
    revalidate();
  }
  
  public void actionPerformed(ActionEvent evt) {
    _app.setPanneau("Config");   // précédent
  }
  
  public void valueChanged(ListSelectionEvent e) {
    if (!e.getValueIsAdjusting()) {
      if (_listeThemes.getSelectedValue() != null) {
        ((TbiCetidem)_app).setThemeSelectionne((Theme)(_listeThemes.getSelectedValue()));
        if (_app.getUtilisationDesClasses()) {
          _app.demarreNouvelleEvaluation(); 
          _app.setPanneau("SelectEleveGen");
        } else  
          _app.setPanneau(_app.getPanneauApresSelectEleve("premier"));
      }
    }
  }
  
}

class ThemeListRenderer extends DefaultListCellRenderer {

  private final static long serialVersionUID = 1L;
  
  private float _hauteurPolice;
  private float _ratio = 1f;
  private int _hauteur;

  public ThemeListRenderer(float hauteurPolice) {
    _hauteurPolice = hauteurPolice;
  }
  
  public void setRatio(float r) {
    _ratio = r;
  }
  
  public void setHauteur(int h) {
    _hauteur = h;
  }
  
  @Override
  public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
    JLabel label = (JLabel) super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
    Font f = label.getFont();
    if (f != null) {
      f = f.deriveFont(_hauteurPolice * _ratio);
      label.setFont(f);
    }
    label.setIcon(((Theme)value).getIcon(new Dimension(_hauteur,_hauteur)));
    return label;
  }
}
