/*
Copyright Caroline et Serge COUDÉ, (06/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/
package net.sergecoude.TbiCetidem;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.net.*;
import java.awt.image.*;
import javax.imageio.*;
import net.sergecoude.TbiGeneral.*;
import net.sergecoude.TbiQuizzSonore.Configuration;

public class PanelTableauJeu extends PanelJeu {

  private final static long serialVersionUID = 1L;
  
  private Vector<LigneExercice> _lesExercices;
  private bouton _finOK;

  public PanelTableauJeu(TbiCetidem app) {
    super(app);
		_pMain.setLayout(null); // définition à faire
		// tous les éléments du plateau sont à insérer dans _pMain
    _pMain.addComponentListener(this);		
		_pEvaluations.setVisible(true);
  }
  
  @Override
  public void init() {
    _pMain.removeAll();
    _pMain.repaint();
    if (_finOK == null)
      try {
		    _finOK = new bouton(_app.getSkin().getRessource("ok"));
		    _finOK.addActionListener(this);
      } catch(Exception e) {
        _app.log(e);
      }
    if (_finOK != null) {
      _finOK.setOpacite(0f);
      _finOK.setVisible(false);
      _pMain.add(_finOK);
      Dimension dim = _pMain.getSize();
      int d = (dim.width > dim.height ? dim.height : dim.width)/2;
      _finOK.setBounds((dim.width-d)/2, (dim.height-d)/2, d, d);
    }  
    _lesExercices = new Vector<LigneExercice>();
    Theme theme = ((TbiCetidem)_app).getThemeSelectionne();
    // Mélange des images modèles et suppression des images équivalentes pour éviter d'avoir la même image pour l'ensemble des exercices
    Vector<Element> lesModelesPossibles = new Vector<Element>();
    for(int i = 0; i < theme.getNbElements(); i++)
      lesModelesPossibles.addElement(theme.elementAt(i));
    Collections.shuffle(lesModelesPossibles);
    boolean tousDifferents = false;
    while(!tousDifferents) {
      for(int i = 0; i < lesModelesPossibles.size()-1; i++) {
        boolean retrait = false; 
        for(int j = i+1; j < lesModelesPossibles.size(); j++) {
          if (theme.verifieCompatibilite(lesModelesPossibles.elementAt(i), lesModelesPossibles.elementAt(j)) == Configuration.CONFIGURATION_TYPE_EQUIVALENCE_ID) {
            retrait = true;
            lesModelesPossibles.removeElementAt(j);
            break;              
          }
          if (i == lesModelesPossibles.size()-2 && j == lesModelesPossibles.size()-1)
            tousDifferents = true; 
        }
        if (retrait)
          break;
      }      
    }
    int nbExercices = _app.getNbExercices() <= lesModelesPossibles.size() ? _app.getNbExercices() : lesModelesPossibles.size();
    Vector<Element> lesModeles = new Vector<Element>();
    for(int i = 0; i < nbExercices; i++) {
      Element eModele = lesModelesPossibles.elementAt((int)(Math.random() * lesModelesPossibles.size()));
      lesModelesPossibles.removeElement(eModele);
      boolean modeleVisible = true;
      if (((TbiCetidem)_app).getVisibiliteModele().equals(TbiCetidem._V_VM_Non) || 
          (((TbiCetidem)_app).getVisibiliteModele().equals(TbiCetidem._V_VM_Aleatoire) && Math.random() >= 0.5))
        modeleVisible = false;
      LigneExercice le = new LigneExercice((TbiCetidem)_app, this, _pMain, eModele, theme, distribueElements(eModele, theme, ((TbiCetidem)_app).getNbImagesProposees(), ((TbiCetidem)_app).getAvecRotation(), ((TbiCetidem)_app).getQuantiteIdentique()), modeleVisible);
      le.setVisible(false);
      _lesExercices.add(le);
      if (i == nbExercices-1)
        le.setDernierExercice(true);
    }
    super.init();
    // on met à jour le nombre d'exercices réellement à faire
    _mireCourante.setNbExercices(_lesExercices.size());
    _init = true;
    nouvelExercice();
  }
  
  private int getNbElementsIdentiques(int nbChoix, String quantIdent) {
    if (quantIdent.equals(TbiCetidem._V_QI_Peu)) {
      switch(nbChoix) {
        case 2 : return 1;
        case 3 : return 1;
        case 4 : return 1;
        case 5 : return 1;
        case 6 : return 2;
        default: break;
      }
    }
    if (quantIdent.equals(TbiCetidem._V_QI_Moyen)) {
      switch(nbChoix) {
        case 2 : return 1;
        case 3 : return 1;
        case 4 : return 2;
        case 5 : return 2;
        case 6 : return 3;
        default: break;
      }
    }
    if (quantIdent.equals(TbiCetidem._V_QI_Beaucoup)) {
      switch(nbChoix) {
        case 2 : return 1;
        case 3 : return 2;
        case 4 : return 3;
        case 5 : return 4;
        case 6 : return 5;
        default: break;
      }
    }
    return 1;    
  }  
 
  private Vector<Element> distribueElements(Element eModele, Theme theme, int nbChoix, boolean rotation, String quantIdent) {
    Vector<Element> tmp = new Vector<Element>(); // les éléments pouvant être choisis une seule fois
    Vector<Element> tmp2 = new Vector<Element>(); // les éléments pouvant être choisis plusieurs fois en cas de défaut du tmp 
    Vector<Element> tmpEqui = new Vector<Element>(); // les éléments étant équivalents au modèle                                                  
    Vector<Element> res = new Vector<Element>();
    for(int i = 0; i < theme.getNbElements(); i++) 
      if (theme.elementAt(i) != eModele) {
        tmp.addElement(theme.elementAt(i));
        tmp2.addElement(theme.elementAt(i));
        if (theme.verifieCompatibilite(eModele, theme.elementAt(i)) == Configuration.CONFIGURATION_TYPE_EQUIVALENCE_ID)
          tmpEqui.addElement(theme.elementAt(i));
      }
    eModele.addActionListener(this);
    int qIdent = getNbElementsIdentiques(nbChoix, quantIdent);
    int i = 0;
    Element elem = null;
    for(; i < qIdent && i < tmpEqui.size(); i++) { // les équivalents
      try {
        elem = tmpEqui.elementAt((int)(Math.random() * tmpEqui.size()));
        tmpEqui.removeElement(elem);      
        elem = (Element)elem.clone();
        elem.addActionListener(this);
        elem.setIdentique(true); 
        res.addElement(elem);
      } catch(Exception e) {
        elem = null;
        _app.log(e);
      }
    }
    for(; i < qIdent; i++) { // les identiques
      try {
        elem = (Element)eModele.clone();
        elem.addActionListener(this);
        elem.setIdentique(true);
        res.addElement(elem);
      } catch(Exception e) {
        elem = null;
        _app.log(e);
      }
    }
    for(; i < nbChoix; i++) { // les autres (avec identique pivoté éventuellement)
      elem = null;
      if (rotation) {
        double rot = Math.random();
        if (rot >= 0.5) {
          try {
            elem = (Element)eModele.clone();
            if (rot <= 0.666666666)
              elem.pivote(90);
            else if (rot <= 0.8333333333)
              elem.pivote(180);
            else 
              elem.pivote(270);
          } catch (Exception e) {
            elem = null;
            _app.log(e);
          }   
        }
      }
      if (elem == null) {
        while(true) {
          // on prend une image qui n'est pas équivalente au modèle
          if (tmp.size() > 0) { 
            elem = tmp.elementAt((int)(Math.random() * tmp.size()));
            if (theme.verifieCompatibilite(eModele, elem) != Configuration.CONFIGURATION_TYPE_EQUIVALENCE_ID)
              break;
          } else { // tmp épuisé, on puise dans tmp2
            elem = tmp2.elementAt((int)(Math.random() * tmp2.size()));
            if (theme.verifieCompatibilite(eModele, elem) != Configuration.CONFIGURATION_TYPE_EQUIVALENCE_ID)
              break;
          }
        }
        if (tmp.size() > 0)
          tmp.removeElement(elem);
        try {
          elem = (Element)elem.clone();
        } catch (Exception e) {
          _app.log(e);
        }
      }
      elem.addActionListener(this);
      elem.setIdentique(false);
 		  res.addElement(elem);
    }
    Collections.shuffle(res);
    return res;
  }
  
  private boolean estUnModele(Element elem) {
    for(int i = 0; i < _lesExercices.size(); i++)
      if (_lesExercices.elementAt(i).estLeModele(elem))
        return true;
    return false;
  }
  
  public void actionPerformed(ActionEvent evt) {
    super.actionPerformed(evt);
		Object source = evt.getSource();
		if (source instanceof Element) {
      Element elem = (Element)source;
      if (((TbiCetidem)_app).getAvecNomPrononce() || estUnModele(elem)) {
        try {
          _app.play(elem.getURLNom().toURI().toURL());
        } catch (MalformedURLException mue) {
          _app.message(mue.toString(), _app.MESSAGE_ERREUR);
        } catch (URISyntaxException use) {
          _app.message(use.toString(), _app.MESSAGE_ERREUR);
        }
      }
      if (!estUnModele(elem)) 
        majExercice(elem, !elem.estCoche());
    }
 		if (source instanceof bouton)
      if (source == _finOK) {
        _pMain.removeAll();
        finExercice(); 
      } else if (estBoutonSuivant((bouton)source)) {
        nouvelExercice();
      } else
        ecouteExercice((bouton)source);
  }
  
  private void majExercice(Element elem, boolean nEtat) {
    for(int i = 0; i < _lesExercices.size(); i++)
      _lesExercices.elementAt(i).majExercice(elem, nEtat);
  }
  
  private boolean estBoutonSuivant(bouton b) {
    for(int i = 0; i < _lesExercices.size(); i++) {
      LigneExercice le = _lesExercices.elementAt(i);
      if (le.pointeBoutonSuivant(b))
        return true;
    }
    return false;
  }

  private void ecouteExercice(bouton b) {
    for(int i = 0; i < _lesExercices.size(); i++) {
      LigneExercice le = _lesExercices.elementAt(i);
      if (le.pointeCetExercice(b)) {
        le.ecoute((TbiCetidem)_app);
        break;
      }  
    }
  }
  
  @Override
  public void agenceLesComposants() {
    super.agenceLesComposants();
    Rectangle dim = _pMain.getBounds();
    _largeurPlateau = dim.width;
    _hauteurPlateau = dim.height;
    int x = 0;
    int l = _largeurPlateau;
    int h = (_hauteurPlateau-_app.getHauteurLogo())/3;
    int y = h;
    for(int i = 0; i < _lesExercices.size(); i++) {
      LigneExercice le = _lesExercices.elementAt(i);
      le.agenceLesComposants(x,y,l,h);
    }
    revalidate();
  }
  
  @Override
  public void nouvelExercice() {
    if (!_init)
      return;
    super.nouvelExercice();
    if (_nbExercices < _app.getNbExercices()) {
      if (_nbExercices > 0)
        _lesExercices.elementAt(_nbExercices-1).setVisible(false);
      if (_app.getUtilisationDesClasses()) 
        if (_app.getEpreuve() != null) 
          if (_app.getEpreuve().getExercice() != null)        
            _lesExercices.elementAt(_nbExercices).setExercice((ExerciceCetidem)(_app.getEpreuve().getExercice())); 
      _lesExercices.elementAt(_nbExercices).setVisible(true);
    }
  }  
  
  @Override
  public boolean verifieFinExercices() {
    if (_nbExercices >= _lesExercices.size()) {
      afficheFinExercice();
      return true; 
    } else 
      return false;
  }

  public void afficheFinExercice() {
    _finOK.faitApparaitre(true, 0, 2000);
  }

}

class LigneExercice {

  private final static long serialVersionUID = 1L;

  private TbiCetidem _app;
  private Element _eModele;
  private Theme _theme;
  private int _nbChoix;
  private bouton _bOK;
  private bouton _bKO;
  private bouton _bSuivant;
  private bouton _pi;
  private JPanel _pModele;
  private JPanel _pChoix;
  private JPanel _pSmileys;
  private JPanel _pMain;
  private Vector<Element> _lesChoix;
  private boolean _exerciceOK = false;
  private PanelTableauJeu _plateau = null;
  private boolean _devoile = false;
  private ExerciceCetidem _exercice;
  private boolean _estDernierExercice = false;
  private boolean _modeleVisible = true;

  public LigneExercice(TbiCetidem app, PanelTableauJeu plateau, JPanel pMain, Element eModele, Theme theme, Vector<Element> lesChoix, boolean modeleVisible) {
    _app = app;
    _plateau = plateau;
    _pMain = pMain;
    _eModele = eModele;
    _theme = theme;
    _lesChoix = lesChoix;
    _nbChoix = lesChoix.size();
    _modeleVisible = modeleVisible;
    try {
		  _bOK = new bouton(app.getSkin().getRessource("ok"));
    } catch(Exception e) {
      app.log(e);
    }
    try {
		  _bKO = new bouton(app.getSkin().getRessource("ko"));
    } catch(Exception e) {
      app.log(e);
    }
    try {
		  _bSuivant = new bouton(app.getSkin().getRessource("suivant"));
		  _bSuivant.addActionListener(plateau);
    } catch(Exception e) {
      app.log(e);
    }
    try {
		  _pi = new bouton(app.getSkin().getRessource("pi"));
		  _pi.addActionListener(plateau);
    } catch(Exception e) {
      app.log(e);
    }
    _pModele = new JPanel();
    //_pModele.setBackground(Color.lightGray);
    _pModele.setBackground(Color.black);
    _pModele.setOpaque(true);
    //_pModele.setBorder(BorderFactory.createLineBorder(Color.black, 3));
    _pChoix = new JPanel(new FlowLayout(FlowLayout.CENTER, 10, 0));
    _pChoix.setOpaque(false);
    _pChoix.setBorder(BorderFactory.createLineBorder(Color.black));
    _pSmileys = new JPanel();
    _pSmileys.setOpaque(false);
    _pSmileys.setBorder(BorderFactory.createLineBorder(Color.black));
    _pMain.add(eModele);
    _pMain.add(_pi);
    _pMain.add(_pModele);
    for(int i = 0; i < lesChoix.size(); i++) {
      Element elem = lesChoix.elementAt(i);
      elem.setVisible(false); 
      _pMain.add(elem);
    }
    _pMain.add(_pChoix);
    _pMain.add(_bSuivant);
    _pMain.add(_bOK);
    _pMain.add(_bKO);
    _pMain.add(_pSmileys);
    _bOK.setVisible(false);
    _bKO.setVisible(false);
    _bSuivant.setVisible(false);
    _eModele.setVisible(false);
  }
  
  public void setDernierExercice(boolean dernier) {
    _estDernierExercice = dernier;
    if (_estDernierExercice) {
      _pMain.remove(_bSuivant);
      _bSuivant = null;
    }
  }
  
  public void agenceLesComposants(int x, int y, int l, int h) {
    int xEspacement = l / 40;
    int nbElements = 1 + _nbChoix + 1; 
    int lElement = (l - ((nbElements+1)*xEspacement)) / nbElements;  
    if (lElement > h) {
      lElement = h;
      xEspacement = (l - (lElement * nbElements)) / (nbElements+1);
    }
    int hElement = lElement;
    _eModele.setBounds(x+xEspacement, y+(h-hElement)/2, lElement, hElement);
    _pi.setBounds(x+xEspacement, y+(h-hElement)/2, lElement, hElement); // identique à _eModele
    _pModele.setBounds(x+xEspacement/2, y+(h-hElement)/4, lElement+xEspacement, hElement+(h-hElement)/2);
    int xElement = x+xEspacement+lElement+xEspacement;
    int yElement = y+(h-hElement)/2;
    int yPanel = y+(h-hElement)/4;  
    for(int i = 0; i < _lesChoix.size(); i++) {
      _lesChoix.elementAt(i).setBounds(xElement, yElement, lElement, hElement);
      xElement += xEspacement + lElement;
    }
    _pChoix.setBounds(x+xEspacement+lElement+xEspacement/2, yPanel, x+l-xEspacement-lElement-xEspacement/2-(xEspacement+lElement+xEspacement/2), (hElement+h)/2);
    _bOK.setBounds(x+l-xEspacement-lElement, yElement, lElement, hElement);
    _bKO.setBounds(x+l-xEspacement-lElement, yElement, lElement, hElement);
    if (_bSuivant != null)
      _bSuivant.setBounds(x+l-xEspacement-lElement, yElement, lElement, hElement);
    _pSmileys.setBounds(l-xEspacement-lElement-xEspacement/2, yPanel, lElement+xEspacement, (hElement+h)/2);
  }
  
  public void devoileExercice() {
    _devoile = true;
    _bKO.setVisible(true);
    if (_modeleVisible) {
      _pi.setVisible(false);
      _eModele.setVisible(true);
    } else {
      _pi.setVisible(true);
      _eModele.setVisible(false);
    }
    for(int i = 0; i < _lesChoix.size(); i++) 
      _lesChoix.elementAt(i).setVisible(true);
  }
  
  public boolean estDevoile() {
    return _devoile;
  }
  
  public void setVisible(boolean visible) {
    _eModele.setVisible(false);
    _bOK.setVisible(false);
    if (_bSuivant != null)
      _bSuivant.setVisible(false);
    _bKO.setVisible(visible);
    _pi.setVisible(visible);
    _pModele.setVisible(visible);
    _pChoix.setVisible(visible);
    _pSmileys.setVisible(visible);
    if (visible) {
      devoileExercice();
    } else {
      for(int i = 0; i < _lesChoix.size(); i++) 
        _lesChoix.elementAt(i).setVisible(false);
    }
  }
  
  public void setExercice(ExerciceCetidem ex) {
    _exercice = ex;
    _exercice.setBonneReponse(_eModele);
    _exercice.setVisibiliteModele(_modeleVisible);
  }
  
  public boolean pointeCetExercice(bouton b) {
    return b == _pi;
  }
  
  public boolean pointeBoutonSuivant(bouton b) {
    return (_bSuivant != null && b == _bSuivant); 
  }
  
  public void majExercice(Element choix, boolean nEtat) {
    if (_lesChoix.contains(choix)) {
      if (_exerciceOK)
        return;
      choix.setCoche(nEtat);
      if (_exercice != null && choix.estCoche()) {
        _exercice.addEnregistrement(choix, (int)(Calendar.getInstance().getTimeInMillis()-_exercice.getDateStart().getTimeInMillis()), choix.identique());
      }
      boolean ok = true;
      for(int i = 0; i < _lesChoix.size(); i++) {
        Element elemI = _lesChoix.elementAt(i);
        if ((elemI.estCoche() && !elemI.identique()) ||
            (!elemI.estCoche() && elemI.identique())) {
          ok = false;
          break;
        }
      }
      if (ok) {
        if (_exercice != null)
          _exercice.stop();
        _bOK.setVisible(true);
        _bKO.setVisible(false);
        _exerciceOK = true;
        _plateau.succes();
        _bOK.faitApparaitre(false,0,2000);
        if (_bSuivant != null)
          _bSuivant.faitApparaitre(true,500,2000);
      } else {
        _bOK.setVisible(false);
        if (_bSuivant != null)
          _bSuivant.setVisible(false);
        _bKO.setVisible(true);          
      }
    }
  }
  
  public boolean fini() {
    return _exerciceOK;
  }
  
  public boolean estLeModele(Element elem) {
    return elem == _eModele;
  }
  
  public void ecoute(TbiCetidem app) {
    try {
      app.play(_eModele.getURLNom().toURI().toURL());
    } catch (MalformedURLException mue) {
      app.message(mue.toString(), _app.MESSAGE_ERREUR);
    } catch (URISyntaxException use) {
      app.message(use.toString(), _app.MESSAGE_ERREUR);
    }
  }

}