/*
Copyright Caroline et Serge COUDÉ, (06/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/
package net.sergecoude.TbiCetidem;

import java.util.*;
import java.io.*;
import javax.swing.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import net.n3.nanoxml.*;
import net.sergecoude.TbiGeneral.*;

public class TbiCetidem extends TbiApp {

  private final static long serialVersionUID = 1L;
	
  // Définition des variables propres à l'application
  static public String _V_NbImagesProposees = "nbimagesproposees";
  private int _nbImagesProposees = 6;
  static public String _V_CheminDesThemes = "chemindesthemes";
  private String _cheminDesThemes = ".";
  static public String _V_NomPrononce = "nomprononce";
  private boolean _avecNomPrononce = true;
  static public String _V_Rotation = "rotation";
  private boolean _avecRotation = false;
  static public String _V_QuantiteIdentique = "quantiteidentique";
  static public String _V_QI_Peu = "peu";
  static public String _V_QI_Moyen = "moyen";
  static public String _V_QI_Beaucoup = "beaucoup";
  private String _quantiteIdentique = _V_QI_Moyen;
  static public String _V_VisibiliteModele = "visibilitemodele";
  static public String _V_VM_Oui = "oui";
  static public String _V_VM_Non = "non";
  static public String _V_VM_Aleatoire = "aleatoire";
  private String _visibiliteModele = _V_VM_Oui;
  
  private PanelSelectTheme _pSelectTheme;
  private DefaultListModel _lesThemes = null;
  private Theme _themeSelectionne = null;
  
	public TbiCetidem(String fichierLog) {
    super(fichierLog);
    _applicationInfo = new ApplicationInfoCetidem();
    setTitle(((ApplicationInfoCetidem)_applicationInfo).getNom() + " " + ((ApplicationInfoCetidem)_applicationInfo).getVersion());
    _fichierConf = getRepertoireApplication()+"/TbiCetidem.conf";
    _nomDuSkin = "skinPetits";
    /*************************************
     * 0 : TbiGeneralApp.conf    
     * 1 : TbiGeneral.conf
     * 2 : TbiCetidemApp.conf     
     * 3 : TbiCetidem.conf
     *************************************/
    if (!_configuration.chargeConfiguration("2", "TbiCetidemApp.conf", false, false))
      System.exit(-1); 
    if (!_configuration.chargeConfiguration("3", _fichierConf, false, false))
      System.exit(-1); 

    _lesThemes = new DefaultListModel();
		_pSelectEcran = new PanelSelectEcran(this);
		_pMain.add(_pSelectEcran,"SelectEcran");
    // enchaînement des panneaux 
		addPanneauApresSelectClasse("SelectTheme"); // à définir obligatoirement
		addPanneauApresSelectEleve("TableauJeu"); // à définir obligatoirement
		_pSelectTheme = new PanelSelectTheme(this);
		_pMain.add(_pSelectTheme,"SelectTheme");
		_pTitre = new PanelTitreCetidem(this);
		_pMain.add(_pTitre,"Titre");
    getContentPane().setBackground(_cFond);
    initSuivantConfiguration(); // obligatoire !
    // tableau jeu et panneau de conf doivent être chargés après initSuivantConfiguration car chargement des skins au préalable...
		_pTableauJeu = new PanelTableauJeu(this);
		_pMain.add(_pTableauJeu,"TableauJeu");
		_pConfig = new PanelConfigCetidem(this);
		_pMain.add(_pConfig,"Config");
		// setPanneau doit être appelée après la création du plateau de jeu (setSouris)
		setPanneau("Titre");
    _timer = new java.util.Timer();
    _timer.schedule(new TimerTask() {
        public void run() {
            setPanneau("SelectEcran");
            stopTimer();
        }
    }, 10000);
		setVisible(true);
    chargeLesThemes();    		
	}
	
	@Override
  public String getRepertoireApplication() {
    return "TbiCetidem";
  }
  
 public void chargeLesThemes() {
    message("Chargement des thèmes...", MESSAGE_INFORMATION, 0);
    _pSelectTheme.setEnabled(false); // on désactive le clic sur la liste
    _lesThemes.clear();
    File[] fichiers = null;
    File rep = new File(getConfiguration("chemindesthemes"));
    fichiers = rep.listFiles();
    if (fichiers != null) {
      for(int f = 0; f < fichiers.length; f++) {
        if (fichiers[f].isDirectory()) {
          File desc = new File(fichiers[f], "description.txt");
          File icon = new File(fichiers[f], "icon.jpg");
          if (desc.exists() && icon.exists()) {
            try {
              Theme theme = new Theme(fichiers[f]);
              _lesThemes.addElement(theme);
            } catch (Exception e) {
                message(e, MESSAGE_AVERTISSEMENT);      
            }
          }
        }
      }
    }
    _pSelectTheme.setEnabled(true); // on résactive le clic sur la liste
    if (_lesThemes.getSize() > 0)
      message("Chargement des thèmes : " + _lesThemes.getSize() + " trouvé" + (_lesThemes.getSize() > 1 ? "s" : "") + " !", MESSAGE_INFORMATION, 3000);
    else 
      message("Chargement des thèmes : aucun thème trouvé...", MESSAGE_AVERTISSEMENT, 3000);
  }   
	
/*	@Override
  protected void quitter() {
    int rep = JOptionPane.showConfirmDialog(_pFrame, "Souhaitez-vous enregistrer les informations du jour ?", "Quitter", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
    if (rep == JOptionPane.NO_OPTION || enregistrerLesEvaluations()) 
      System.exit(0);
  }*/
  
  @Override
  public void initSuivantConfiguration() {
    super.initSuivantConfiguration();
    setNbExercices(Integer.parseInt(getConfiguration(_V_NbExercices))); // obligatoire 
    setAvecNomPrononce(getConfiguration(_V_NomPrononce).toLowerCase().equals("oui"));
    setNbImagesProposees(Integer.parseInt(getConfiguration(_V_NbImagesProposees)));
    setAvecRotation(getConfiguration(_V_Rotation).toLowerCase().equals("oui"));
    setQuantiteIdentique(getConfiguration(_V_QuantiteIdentique));
    setVisibiliteModele(getConfiguration(_V_VisibiliteModele));
    setCheminDesThemes(getConfiguration(_V_CheminDesThemes));    
  }
	
	@Override
	public void setPanneau(String p) {
	  if (!p.equals("TableauJeu")) {
      setCursor(Cursor.getDefaultCursor());
      _pTableauJeu.setSouris(false);
    }
    super.setPanneau(p);
  }
	
    public Theme getThemeSelectionne() {
    return _themeSelectionne;
  }

  public void setThemeSelectionne(Theme theme) {
    _themeSelectionne = theme;
  }
  
  public int getNbImagesProposees() {
/*    if (_themeSelectionne != null)
      if (_themeSelectionne.getNbElements() < _nbImagesProposees) {
        return _themeSelectionne.getNbElements();
      }*/
    return _nbImagesProposees;
  }

  public void setNbImagesProposees(int n) {
    _nbImagesProposees = n;
  }

  public void setAvecNomPrononce(boolean oui) {
    _avecNomPrononce = oui;
  }

  public boolean getAvecNomPrononce() {
    return _avecNomPrononce;
  }

  public boolean getAvecRotation() {
    return _avecRotation;
  }

  public void setAvecRotation(boolean oui) {
    _avecRotation = oui;
  }

  public String getQuantiteIdentique() {
    return _quantiteIdentique;
  }

  public void setQuantiteIdentique(String v) {
    _quantiteIdentique = v;
  }

  public String getVisibiliteModele() {
    return _visibiliteModele;
  }

  public void setVisibiliteModele(String v) {
    _visibiliteModele = v;
  }
  
  public String getCheminDesThemes() {
    return _cheminDesThemes;
  }

  public void setCheminDesThemes(String chemin) {
    _cheminDesThemes = chemin;
  }
  
  @Override
  public boolean sauveConfiguration() {
    /*************************************
     * 0 : TbiGeneralApp.conf    
     * 1 : TbiGeneral.conf
     * 2 : TbiCetidemApp.conf     
     * 3 : TbiCetidem.conf
     *************************************/
    setConfiguration("3", _V_NbImagesProposees, Integer.toString(_nbImagesProposees));
    setConfiguration("3", _V_NomPrononce, _avecNomPrononce ? "oui" : "non");
    setConfiguration("3", _V_CheminDesThemes, _cheminDesThemes);
    setConfiguration("3", _V_Rotation, _avecRotation ? "oui" : "non");
    setConfiguration("3", _V_TailleSouris, Integer.toString(_tailleCurseur));
    setConfiguration("3", _V_UtilisationTBI, _avecTbi ? "oui" : "non");
    setConfiguration("3", _V_LargeurTBI, Integer.toString(_largeurTbi));    
    setConfiguration("3", _V_HauteurTBI, Integer.toString(_hauteurTbi));
    setConfiguration("3", _V_CoordXTBI, Integer.toString(_xTbi));    
    setConfiguration("3", _V_CoordYTBI, Integer.toString(_yTbi));
    setConfiguration("3", _V_UtilisationDesClasses, _utilisationDesClasses ? "oui" : "non");
    setConfiguration("3", _V_NbExercices, Integer.toString(_nbExercices));
    setConfiguration("3", _V_NomDuSkin, getNomDuSkin());
    setConfiguration("3", _V_QuantiteIdentique, _quantiteIdentique);
    setConfiguration("3", _V_VisibiliteModele, _visibiliteModele);        
    return sauveConfiguration("3"); // fichier TbiCetidem.conf
  }
 
  public Epreuve getNewEpreuve() {
    return new EpreuveCetidem(this);
  }
  
  public Epreuve getNewEpreuve(Classe classe, IXMLElement xml) throws Exception {
    return new EpreuveCetidem(this, classe, xml);
  }
  
  public Evaluation getNewEvaluation() {
    EvaluationCetidem eval = new EvaluationCetidem(this);
    eval.setNbImages(getNbImagesProposees());
    eval.setQuantiteIdentique(getQuantiteIdentique());
    eval.setRotation(getAvecRotation());
    eval.setVisibiliteModele(getVisibiliteModele());
    return eval;
  }

  @Override
  public String genereCleEvaluation() {
    if (getThemeSelectionne() != null) 
      return super.genereCleEvaluation() + getThemeSelectionne().getNom() + "-" + getVisibiliteModele() + "-" + getNbImagesProposees() + "-" + getQuantiteIdentique() + "-" + (getAvecRotation() ? "1" : "0");
    return super.genereCleEvaluation() + "-" + "-" + getVisibiliteModele() + "-" + getNbImagesProposees() + "-" + getQuantiteIdentique() + "-" + (getAvecRotation() ? "1" : "0");
  }

  @Override
  public String genereNomEvaluation() {
    return getThemeSelectionne().getNom();
  }

  public Evaluation getNewEvaluation(IXMLElement xml) throws Exception {
    EvaluationCetidem eval = new EvaluationCetidem(this);
    eval.chargeEvaluation(xml);
    return eval;
  }
  
  public Evaluation getNewEvaluation(String fichier) throws Exception {
    return new EvaluationCetidem(this, fichier);
  } 
  
  @Override
  public void selectionneEleve(Eleve e, boolean oui) {
    if (getAvecNomPrononce()) {
        play(e.getItemEleve().getAudio());
    }
    super.selectionneEleve(e, oui);
  }
  
  public DefaultListModel getLesThemes() {
    return _lesThemes;
  }  
  
	public static void main(String[] args) {
 	  try {
      new TbiCetidem("TbiCetidem.log");
	  } catch (Exception e) {
      Log.logStatic(e);
    }
	}
}
