/*
Copyright Caroline et Serge COUDÉ, (01/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/

package net.sergecoude.TbiGeneral;

import java.util.*;
import java.util.zip.*;
import java.io.*;
import net.n3.nanoxml.*;
import java.nio.channels.*;
import net.sergecoude.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class Classe implements IObjetAvecItem {

  private final static long serialVersionUID = 1L;

  private ArrayList<Eleve> _lesEleves = null;
  private String _fichierZip = "";
  private String _nom = "";
  private TbiApp _app = null;
  private String _id = "";
  private boolean _estEnregistree = false;
  
  public static int NOM_ASC = 0;
  public static int NOM_DESC = 1; 
  private static ArrayList<ParametreDeTri> _lesParametresDeTri = null;
  
  private ItemClasseGen _itemClasse = null;

  public Classe(TbiApp app) {
    _app = app;
    _lesEleves = new ArrayList<Eleve>();
    _fichierZip = null;
    _itemClasse = _app.genereItemClasse(this);
    _id = "";    
  }
  
  public Classe(TbiApp app, ArrayList<Eleve> l) {
    this(app);
    _lesEleves = l;
  }

  public Classe(TbiApp app, File f) {
    this(app);
   _fichierZip = f.getAbsolutePath();
   charger(_fichierZip);
  }
  
  public TbiApp getApplication() {
    return _app;
  }
  
  public void add(Eleve e) {
    if (!_lesEleves.contains(e))
      _lesEleves.add(e);
    _itemClasse.setNbEleves(_lesEleves.size());
  }
  
  public void remove(Eleve e) {
    _lesEleves.remove(_lesEleves.indexOf(e));
    _itemClasse.setNbEleves(_lesEleves.size());
  }
  
  public Eleve elementAt(int index) {
    return _lesEleves.get(index);
  }
  
  public ArrayList<Eleve> getLesEleves() {
    return _lesEleves;
  }
  
  public Eleve getEleveById(String id) {
    for(int i = 0; i < _lesEleves.size(); i++) 
      if (_lesEleves.get(i).getId().equals(id))
        return _lesEleves.get(i);
    return null;  
  }
  
  public ItemClasseGen getItemClasse() {
    return _itemClasse;
  }
  
  public JPanel getItem() {
    return getItemClasse();
  }  
  
  public int size() {
    return _lesEleves.size();
  }
  
  public String getFichier() {
    return _fichierZip;
  }
  
  public void setFichier(String fichier) {
    _fichierZip = fichier; 
  }

  public String getNom() {
    return _nom;
  }
  
  public void setNom(String nom) {
    if (nom != null)
      nom = nom.trim();
    _nom = nom;
    _itemClasse.setNom(_nom);
  }
  
  public String getId() {
    return _id;
  }
  
  public void setId(String id) {
    if (id != null)
      id = id.trim();
    _id = id;
  }
  
  public ArrayList<Eleve> tri(Comparator<Eleve> cp) {
    ArrayList<Eleve> liste = new ArrayList<Eleve>();
    for(int i = 0; i < _lesEleves.size(); i++)
      liste.add(_lesEleves.get(i));
    Collections.sort(liste, cp);
    return liste;
  }
	
	private boolean stockeFichier(ZipOutputStream out, String cheminFichier) throws IOException {
	  FileInputStream fis = null;
    BufferedInputStream buffi = null;
    ZipInputStream zis = null;
    byte[] buf = new byte[1024];
    if (cheminFichier != null && !"".equals(cheminFichier)) {
      if (cheminFichier.indexOf("file:") >= 0) {
        // fichier déjà dans le zip initialement ouvert
        String nomFichier = cheminFichier.substring(5); 
        boolean trouve = false;
        // on vérifie qu'il est bien présent dans le zip
        fis = new FileInputStream(_fichierZip);
        buffi = new BufferedInputStream(fis);
        zis = new ZipInputStream(buffi);
        ZipEntry entry;
        while((entry = zis.getNextEntry()) != null) {
          if (entry.getName().equals(nomFichier)) {
            trouve = true;
            out.putNextEntry(new ZipEntry(nomFichier));
            int len;
            while ((len = zis.read(buf, 0, 1024)) > 0) {
              out.write(buf, 0, len);
            }                      
            out.closeEntry();                      
            break;                                                                
          }
        }
        if (!trouve) {
          TbiApp.messageStatique("Impossible de trouver le fichier " + cheminFichier, TbiApp.MESSAGE_ERREUR);
          out.close();
          zis.close();
          return false; 
        }
      } else {
        // fichier local dans le filesystem
        FileInputStream in = new FileInputStream(cheminFichier);
        String nomFichier = cheminFichier.substring(cheminFichier.lastIndexOf(File.separator)+1);
        out.putNextEntry(new ZipEntry(nomFichier));
        int len;
        while ((len = in.read(buf)) > 0) {
          out.write(buf, 0, len);
        }
        out.closeEntry();
        in.close();
      }
    }
    if (zis != null) {
      zis.close();
    } 
    return true;
  }

  public boolean enregistrer(String chemin, String nom, String id) {
    setNom(nom);
    setId(id);
    String ctnClasse = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\" ?>\n";
    ctnClasse += "<classe>\n";
    ctnClasse += "\t<nom>" + nom + "</nom>\n";
    ctnClasse += "\t<id>" + id + "</id>\n";
    ctnClasse += "\t<nb>" + size() + "</nb>\n";
    ctnClasse += "\t<liste>\n";
    ArrayList<String> listePhotos = new ArrayList<String>();
    ArrayList<String> listeAudios = new ArrayList<String>();
    for(int i = 0; i < _lesEleves.size(); i++) {
		    Eleve e = _lesEleves.get(i);
		    ctnClasse += "\t\t" + e.toXml() + "\n";
		    listePhotos.add(e.getInterneCheminImage());
		    listeAudios.add(e.getInterneCheminAudio());
		}
		ctnClasse += "\t</liste>\n";
		ctnClasse += "</classe>\n";
    try {
    		File fichierTmp = File.createTempFile("classe",".tmp");
        fichierTmp.deleteOnExit();
        ZipOutputStream out = new ZipOutputStream(new FileOutputStream(fichierTmp));
        // fichier classe.xml
        out.putNextEntry(new ZipEntry("classe.xml"));
        byte[] tabCtnClasse = ctnClasse.getBytes("UTF-8"); 
        out.write(tabCtnClasse, 0, tabCtnClasse.length);
        out.closeEntry();
        // les photos
        for (int i = 0; i < listePhotos.size(); i++) {
            stockeFichier(out,listePhotos.get(i));
        }
        // les audios
        for (int i = 0; i < listeAudios.size(); i++) {
            stockeFichier(out,listeAudios.get(i));
        }
        out.close();
        _fichierZip = chemin;
        _estEnregistree = true;
        return copieFichierTmp(fichierTmp, _fichierZip);
    } catch (IOException e) {
      _app.log(e);
      return false;
    }
  }
  
  private boolean copieFichierTmp(File fTmp, String chemin) {
    FileChannel in = null; 
    FileChannel out = null;
    if (!chemin.endsWith("-classe.zip")) {
      int posDP = chemin.lastIndexOf(".");
      chemin = chemin.substring(0,posDP) + "-classe.zip";
    }
    boolean ret = false;
    try {
      in = new FileInputStream(fTmp).getChannel();
      out = new FileOutputStream(chemin).getChannel();
      in.transferTo(0, in.size(), out);
      ret = true;
    } catch (Exception e) {
      _app.log(e);
    } finally { // finalement on ferme
      if(in != null) {
        try {
          in.close();
	      } catch (IOException e) {
          _app.log(e);
        }
      }
      if(out != null) {
        try {
          out.close();
        } catch (IOException e) {
          _app.log(e);
        }
      }
    }
    return ret;    
  }
  
  public boolean charger(String fichierZip) {
    _fichierZip = fichierZip;
    try  {
      FileInputStream fis = new FileInputStream(_fichierZip);
      BufferedInputStream buffi = new BufferedInputStream(fis);
      ZipInputStream zis = new ZipInputStream(buffi);    
      ZipEntry entry;
      byte[] buf = new byte[1024];
      String ctnClasseXML = "";
      while((entry = zis.getNextEntry()) != null) {
        if (entry.getName().equals("classe.xml")) {
        	IXMLParser parser = XMLParserFactory.createDefaultXMLParser();
			    IXMLReader reader = new StdXMLReader(zis);
			    parser.setReader(reader);
			    IXMLElement xml = (IXMLElement) parser.parse(); // <classe>
			    IXMLElement xNom = xml.getFirstChildNamed("nom");
			    if (xNom != null)
            setNom(xNom.getContent());
			    IXMLElement xId = xml.getFirstChildNamed("id");
			    if (xId != null)
            setId(xId.getContent());
          IXMLElement xListe = xml.getFirstChildNamed("liste");
			    if (xListe != null) {
            Vector le = xListe.getChildrenNamed("eleve");
            if (le != null) {                       
		          IXMLElement xNb = xml.getFirstChildNamed("nb"); 
		          if (xNb != null && Integer.parseInt(xNb.getContent()) != le.size()) {
		            _app.message("\nIncohérence dans le nombre d'élèves de la classe '" + _nom + "'...\n",_app.MESSAGE_ERREUR);
			          zis.close();		          
		            return false;
              }
    		      _lesEleves.clear();
		          for(int i = 0; i < le.size(); i++) {
  				      IXMLElement e = (IXMLElement)(le.elementAt(i));
  				      add(new Eleve(_app, this, e));
  			      }
  			    } else {
              _app.message("\nPas d'élève présents dans la classe '" + _nom + "'...\n",_app.MESSAGE_AVERTISSEMENT);
            }
  			  } else {
            _app.message("\nPas d'élève présents dans la classe '" + _nom + "'...\n",_app.MESSAGE_AVERTISSEMENT);  			  
          }  
			    zis.close();   
			    break;
			  }
			}
    } catch(FileNotFoundException exc) {
      _app.message("\n" + fichierZip + " n'est pas présent...\n",_app.MESSAGE_ERREUR);
      _app.log(exc);
		  return false;
		} catch(net.n3.nanoxml.XMLException xe) {
      _app.message("\n le fichier classe.xml de l'archive " + fichierZip + " n'est pas valide...\n", _app.MESSAGE_ERREUR);
      _app.log(xe);
      return false;
    } catch(IOException ioe) {
      _app.message("\nErreur de lecture du fichier classe.xml de l'archive " + fichierZip + "...\n", _app.MESSAGE_ERREUR);
      _app.log(ioe);
      return false;      
    } catch(Exception e) {
      _app.message("\nProblème à l'utilisation de la classe de parsage XML...\n", _app.MESSAGE_ERREUR);
      _app.log(e);
      return false;      
    }
    _estEnregistree = true;
    return true;
  }
  
  public ImageIcon getPhoto(Eleve eleve) {
    String chemin = eleve.getCheminImage();
    try  {
      FileInputStream fis = new FileInputStream(_fichierZip);
      BufferedInputStream buffi = new BufferedInputStream(fis);
      ZipInputStream zis = new ZipInputStream(buffi);    
      ZipEntry entry;
      byte[] buf = new byte[1024];
      String ctnClasseXML = "";
      while((entry = zis.getNextEntry()) != null) {
        if (entry.getName().equals(chemin)) {
          ByteArrayOutputStream out = new ByteArrayOutputStream();
          int i;
          while ((i = zis.read(buf, 0, buf.length)) > 0) {
            out.write(buf, 0, i);
          }
			    zis.close();
			    out.close();
			    ImageIcon img = new ImageIcon(out.toByteArray());
          return img; 
			  }
			}
    } catch(FileNotFoundException exc) {
      _app.message("\n" + _fichierZip + " n'est pas présent...\n",_app.MESSAGE_ERREUR);
      _app.log(exc);
		  return null;
    } catch(IOException ioe) {
      _app.message("\nErreur de lecture de la photo " + chemin + " dans l'archive " + _fichierZip + "...\n", _app.MESSAGE_ERREUR);
      _app.log(ioe);
      return null;      
    } catch(Exception e) {
      _app.message("\nProblème lors de la lecture de l'archive " + _fichierZip + "...\n", _app.MESSAGE_ERREUR);
      _app.log(e);
      return null;      
    }
    return null;
  }

  public ByteArrayInputStream getAudio(Eleve eleve) {
    String chemin = eleve.getCheminAudio();
    try  {
      FileInputStream fis = new FileInputStream(_fichierZip);
      BufferedInputStream buffi = new BufferedInputStream(fis);
      ZipInputStream zis = new ZipInputStream(buffi);    
      ZipEntry entry;
      byte[] buf = new byte[1024];
      String ctnClasseXML = "";
      while((entry = zis.getNextEntry()) != null) {
        if (entry.getName().equals(chemin)) {
          ByteArrayOutputStream out = new ByteArrayOutputStream();
          int i;
          while ((i = zis.read(buf, 0, buf.length)) > 0) {
            out.write(buf, 0, i);
          }
			    zis.close();
			    out.close();
			    return new ByteArrayInputStream(out.toByteArray());
			  }
			}
    } catch(FileNotFoundException exc) {
      _app.message("\n" + _fichierZip + " n'est pas présent...\n",_app.MESSAGE_ERREUR);
      _app.log(exc);
		  return null;
    } catch(IOException ioe) {
      _app.message("\nErreur de lecture du fichier audio " + chemin + " dans l'archive " + _fichierZip + "...\n", _app.MESSAGE_ERREUR);
      _app.log(ioe);
      return null;      
    } catch(Exception e) {
      _app.message("\nProblème lors de la lecture de l'archive " + _fichierZip + "...\n", _app.MESSAGE_ERREUR);
      _app.log(e);
      return null;      
    }
    return null;
  }
  
  public String toString() {
    return getNom();
  }
  
  public boolean estEnregistree() {
    return _estEnregistree;
  }
  
  public boolean verifieUniciteIdEleve(Eleve eleve, String id) {
    Iterator it = _lesEleves.iterator();
    while (it.hasNext()) {
      Eleve e = (Eleve)it.next();
      if (e.getId().equals(id) && eleve != e)
        return false;
    }
    return true;
  }
  
  public boolean verifieUniciteNomUsuelEleve(Eleve eleve, String nomUsuel) {
    Iterator it = _lesEleves.iterator();
    while (it.hasNext()) {
      Eleve e = (Eleve)it.next();
      if (e.getNomUsuel().equals(nomUsuel) && eleve != e)
        return false;
    }
    return true;
  }   
  
  public static ArrayList<ParametreDeTri> getParametresDeTri() {
    if (_lesParametresDeTri == null) {
      _lesParametresDeTri = new ArrayList<ParametreDeTri>();
      _lesParametresDeTri.add(new ParametreDeTri(NOM_ASC, "Nom A->Z"));
      _lesParametresDeTri.add(new ParametreDeTri(NOM_DESC, "Nom Z->A"));
    }
    return _lesParametresDeTri;
  } 

  public static Comparator<Classe> getComparator(ArrayList<ParametreDeTri> paramTri) {
    return new ClasseComparator(paramTri);
  }
  
  private static class ClasseComparator implements Comparator<Classe> {
    private ArrayList<ParametreDeTri> _parametres;

    private ClasseComparator(ArrayList<ParametreDeTri> parametres) {
      _parametres = parametres;
    }

    public int compare(Classe c1, Classe c2) {
      int comparaison = 0;
      for(int i = 0; i < _parametres.size(); i++ ) {
        int idparam = _parametres.get(i).getId();
        if (idparam == Classe.NOM_ASC)
          comparaison = c1.getNom().compareTo(c2.getNom());
        if (idparam == Classe.NOM_DESC)
          comparaison = c2.getNom().compareTo(c1.getNom());
        if (comparaison != 0) return comparaison;
      }
      return 0;
    }
  }
      
}