/*
Copyright Caroline et Serge COUDÉ, (02/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/

package net.sergecoude.TbiGeneral;

import java.util.*;
import net.n3.nanoxml.*;

public class Epreuve  {

  private final static long serialVersionUID = 1L;
  
  public static int DATE_ASC = 0;
  public static int DATE_DESC = 1; 
  private static ArrayList<ParametreDeTri> _lesParametresDeTri;
  
  protected TbiApp _app;
  protected Calendar _dateStart;
  protected Calendar _dateStop;
  protected Eleve _eleve;
  protected Vector<Exercice> _lesExercices; 

  public Epreuve(TbiApp app) {
    _app = app;
    _lesExercices = new Vector<Exercice>();
  }

  public Epreuve(TbiApp app, Classe classe, IXMLElement xml) throws Exception {
    this(app);
		IXMLElement xDebut = xml.getFirstChildNamed("datedebut");
		if (xDebut != null) {
		  _dateStart = Calendar.getInstance();
		  _dateStart.setTimeInMillis(Long.parseLong(xDebut.getContent()));
    } else {
      throw new Exception("La date de début d'épreuve n'est pas indiquée...");
    }    
		IXMLElement xFin = xml.getFirstChildNamed("datefin");
		if (xFin != null) {
		  _dateStop = Calendar.getInstance();
		  _dateStop.setTimeInMillis(Long.parseLong(xFin.getContent()));
    } else {
      throw new Exception("La date de fin d'épreuve n'est pas indiquée...");
    }    
    IXMLElement xEleve = xml.getFirstChildNamed("eleve");
		if (xEleve != null) {
		  _eleve = classe.getEleveById(xEleve.getContent());
		  if (_eleve == null)
		    throw new Exception("L'élève de l'épreuve n'est pas présent dans la classe...");
    } else {
      throw new Exception("L'élève de l'épreuve n'est pas indiqué...");
    }    
    IXMLElement xExercices = xml.getFirstChildNamed("exercices");
		if (xExercices != null) {
      Vector le = xExercices.getChildrenNamed("exercice");
      if (le != null) {
        for(int i = 0; i < le.size(); i++) {
          IXMLElement e = (IXMLElement)(le.elementAt(i));
          getNewExercice(e);
  			}
  		} else 
        throw new Exception("Aucun exercice trouvé dans l'épreuve...");  			  
    }
  }
  
  public void setEleve(Eleve e) {
    _eleve = e;
  }
  
  public Eleve getEleve() {
    return _eleve;
  }
  
  public Exercice getExercice() {
    if (_lesExercices.size() > 0)
      return _lesExercices.lastElement();
    return null;
  }
  
  public Vector<Exercice> getLesExercices() {
    return _lesExercices;
  }
  
  public Exercice getNewExercice() {
    _lesExercices.add(new Exercice(_eleve, _lesExercices.size()+1));
    return getExercice();
  }

  public Exercice getNewExercice(IXMLElement xml) throws Exception {
    _lesExercices.add(new Exercice(_eleve, xml));
    return getExercice();
  }
  
  public void setSucces(int numExercice) {
  }
  
  public void setEchec(int numExercice) {
  }
  
  public boolean estStopee() {
    return _dateStop != null;
  }
  
  public void start() {
    _dateStart = Calendar.getInstance();
  }
  
  public void stop() {
    if (getExercice() != null)
      getExercice().stop();
    _dateStop = Calendar.getInstance();
  }
  
  public String toXmlDebut() {
    String xml = "<epreuve>";
    xml += "<datedebut>" + _dateStart.getTimeInMillis()  + "</datedebut>";
    xml += "<datefin>" + _dateStop.getTimeInMillis() + "</datefin>";
    xml += "<eleve>" + _eleve.getId() + "</eleve>";
    return xml;
  }

  public String toXmlExercices() {
    String xml = "<exercices>";
    for(int e = 0; e < _lesExercices.size(); e++)
      xml += _lesExercices.elementAt(e).toXml();
    xml += "</exercices>";
    return xml;
  }
  
  public String toXmlFin() {    
    return "</epreuve>";
  }
  
  public String toXml() {
    return toXmlDebut() + toXmlExercices() + toXmlFin();
  }
  
  public int getNbSucces() {
    int nbSucces = 0;
    if (_lesExercices != null) {
      for(int i = 0; i < _lesExercices.size(); i++) {
        Exercice ex = _lesExercices.elementAt(i);
        if (ex.estUnSucces())
          nbSucces++;
      }
    }
    return nbSucces;
  }

  public Calendar getDateStart() {
    return _dateStart;
  }

  public Calendar getDateStop() {
    return _dateStop;
  }
  
  public static ArrayList<ParametreDeTri> getParametresDeTri() {
    if (_lesParametresDeTri == null) {
      _lesParametresDeTri = new ArrayList<ParametreDeTri>();
      _lesParametresDeTri.add(new ParametreDeTri(DATE_ASC, "Date ancienne->récente"));
      _lesParametresDeTri.add(new ParametreDeTri(DATE_DESC, "Date récente->ancienne"));
    }
    return _lesParametresDeTri;
  } 

  public static Comparator<Epreuve> getComparator(ArrayList<ParametreDeTri> paramTri) {
    return new EpreuveComparator(paramTri);
  }
  
  private static class EpreuveComparator implements Comparator<Epreuve> {
    private ArrayList<ParametreDeTri> _parametres;

    private EpreuveComparator(ArrayList<ParametreDeTri> parametres) {
      _parametres = parametres;
    }

    public int compare(Epreuve e1, Epreuve e2) {
      int comparaison = 0;
      for(int i = 0; i < _parametres.size(); i++ ) {
        int idparam = _parametres.get(i).getId();
        if (idparam == Epreuve.DATE_ASC)
          comparaison = (e1.getDateStart().getTimeInMillis() < e2.getDateStart().getTimeInMillis() ? -1 : (e1.getDateStart().getTimeInMillis() == e2.getDateStart().getTimeInMillis() ? 0 : 1));
        if (idparam == Epreuve.DATE_DESC)
          comparaison = (e1.getDateStart().getTimeInMillis() < e2.getDateStart().getTimeInMillis() ? 1 : (e1.getDateStart().getTimeInMillis() == e2.getDateStart().getTimeInMillis() ? 0 : -1));
        if (comparaison != 0) return comparaison;
      }
      return 0;
    }
  }  
    
}
