/*
Copyright Caroline et Serge COUDÉ, (11/2011) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/

package net.sergecoude.TbiGeneral;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.awt.image.*;
import java.util.*;

public class Message extends JPanel {

  private final static long serialVersionUID = 1L;

  private TbiApp _app;
  private Font _police = new Font("SansSerif", Font.BOLD, 30);
  private ResizeLabelFont _texte;
  private ImageIcon _iError, _iWarn, _iInfo;
  private java.util.Timer _timer;
	
  public Message(TbiApp app) {
    _app = app;
    setLayout(new BorderLayout());
    _texte = new ResizeLabelFont("");
    _texte.setFont(_police);
    add(_texte, BorderLayout.CENTER);
    setOpaque(false);
    try {
		  _iError = new ImageIcon(_app.getRessource("/icon_erreur.png"));
		  _iWarn = new ImageIcon(_app.getRessource("/icon_avertissement.png"));
		  _iInfo = new ImageIcon(_app.getRessource("/icon_information.png"));
		} catch (Exception e) {
		  _app.log(e);
    }
  }
  	
	public void stopTimer() {
	  if (_timer != null)
      _timer.cancel();
    _timer = null;
    _texte.setIcon(null);
    _texte.setText("");
  }
  
  public void setMessage(String m, int type) {
    setMessage(m, type, 3000);
  }

  public void setMessage(String m, int type, int duree) {
    stopTimer();
    if ("".equals(m))
      return;
    _texte.setText(m);
    Dimension dim = getSize();
    if (type == _app.MESSAGE_ERREUR) 
      _texte.setIcon(getIcon(_iError, dim.height));    
    if (type == _app.MESSAGE_AVERTISSEMENT) 
      _texte.setIcon(getIcon(_iWarn, dim.height));
    if (type == _app.MESSAGE_INFORMATION) 
      _texte.setIcon(getIcon(_iInfo, dim.height));
    repaint();
    if (duree > 0) {
      _timer = new java.util.Timer();
      _timer.schedule(new TimerTask() {
        public void run() {
            stopTimer();
        }
      }, duree);
    }
  }
  
  private ImageIcon getIcon(ImageIcon img, int hw) {
    if (hw == 0)
      return img;
    Image newImg = img.getImage();
    BufferedImage bi = new BufferedImage(hw, hw, BufferedImage.TYPE_INT_ARGB);
    Graphics g = bi.createGraphics();
    g.drawImage(newImg, 0, 0, hw, hw, null);
    return new ImageIcon(bi);
  }
  

}  
