/*
Copyright Caroline et Serge COUDÉ, (02/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce code source est adapté du fichier OggMusic.java écrit par David Karnok, 2012 
dans le cadre du projet "Open Imperium Galactica".
http://code.google.com/p/open-ig/source/browse/trunk/open-ig/src/hu/openig/music/OggMusic.java
  
Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/

package net.sergecoude.TbiGeneral;

import java.io.*;
import javax.sound.sampled.*;
import com.jcraft.jogg.*;
import com.jcraft.jorbis.*;
import java.net.*;

public class OggPlayer extends Thread implements LineListener  {

  private SyncState oy;
  private StreamState os;
  private Page og;
  private Packet op;
  private Info vi;
  private Comment vc;
  private DspState vd;
  private Block vb;
  private static final int BUFSIZE = 128 * 2;
  private int convsize = BUFSIZE * 2;
  private byte[] convbuffer = new byte[convsize];
  private int rate = 0;
  private int channels = 0;
  public volatile SourceDataLine outputLine = null;
  private int initialVolume;
  private BufferedInputStream  _bis;
  private TbiApp _app;
  private URL _url;
  private boolean _isPlaying = false;
  private boolean _startPlaying = false;
  private boolean _pret = true;

  public OggPlayer(TbiApp app) {
    _app = app;
    start();
  }

  public OggPlayer(TbiApp app, BufferedInputStream bis, int volume) {
    this(app);
    playAudioStream(bis,volume);
  }
  
  public void playAudioStream(BufferedInputStream bis, int volume) {
    _bis = bis;
    initialVolume = volume;
    _startPlaying = true;
  }
  
  public void setUrl(URL url) {
    _url = url;
  }
  
  public synchronized void close(boolean arretBrutal) {
    _isPlaying = false;
    if (os != null) { os.clear(); os = null; }
    if (vb != null) { vb.clear(); vb = null; }
    if (vd != null) { vd.clear(); vd = null; }
    if (vi != null) { vi.clear(); vi = null; }
    if (oy != null) { oy.clear(); oy = null; }
    try {  
      if (_bis != null) {
        _bis.close();
        _bis = null;
      }
    } catch (Exception e) {
    }
    
    if (outputLine != null) {
      if (!arretBrutal)
        outputLine.drain();
      outputLine.stop();
      outputLine.flush();
      outputLine.close();
      outputLine = null;
    }      
    _pret = true;
  }

  void initJOrbis() {
    oy = new SyncState();
    os = new StreamState();
    og = new Page();
    op = new Packet();
    vi = new Info();
    vc = new Comment();
    vd = new DspState();
    vb = new Block(vd);

    oy.init();
  }
  
  public void update(LineEvent event) {
/*    if (event.getType() == LineEvent.Type.STOP)
      close(); */      
  } 

  void initAudio(int channels, int rate) {
    try {
      AudioFormat audioFormat = new AudioFormat(rate, 16,
                                                channels, true, // PCM_Signed
                                                false // littleEndian
                                );
      DataLine.Info info = new DataLine.Info(SourceDataLine.class,
                                             audioFormat, AudioSystem.NOT_SPECIFIED);
      if (!AudioSystem.isLineSupported(info)) {
        _app.message("ligne " + info + " non supportée...", TbiApp.MESSAGE_ERREUR);
        return;
      }
      try {
        outputLine = (SourceDataLine) AudioSystem.getLine(info);
        outputLine.addLineListener(this);
        outputLine.open(audioFormat);
      } catch (LineUnavailableException ex) {
        _app.message("Impossible d'ouvrir la ligne audio : " + ex, TbiApp.MESSAGE_ERREUR);
        return;
      } catch (IllegalArgumentException ex) {
        _app.message("Argument illégal : " + ex, TbiApp.MESSAGE_ERREUR);
        return;
      }
      this.rate = rate;
      this.channels = channels;
    } catch (Exception ee) {
      _app.message(ee, TbiApp.MESSAGE_ERREUR);
    }
  }

  SourceDataLine getOutputLine(int channels, int rate) {
    if (outputLine == null || this.rate != rate
                           || this.channels != channels) {
      if (outputLine != null) {
        outputLine.close();
      }
      initAudio(channels, rate);
      outputLine.start();
    }
    return outputLine;
  }
  
  public boolean pret() {
    return _pret;
  }
  
  public void run() {
    boolean chained = false;
    int bytes = 0;
    while(true) {
      try {
        this.sleep(50);
      } catch(Exception exep) {
      }
      if (_bis != null) {
        if (_isPlaying) {
    loop: while (_isPlaying) {
      int eos = 0;
      int index = oy.buffer(BUFSIZE);
      byte[] buffer = oy.data;
      try {
        bytes = _bis.read(buffer, index, BUFSIZE);
      } catch (Exception e) {
        _app.message(e, TbiApp.MESSAGE_ERREUR);
        _pret = true;
        return;
      }
      oy.wrote(bytes);
      if (chained) { 
        chained = false;    
      } else { 
        if (oy.pageout(og) != 1) {
          if (bytes < BUFSIZE) {
            break;
          }
          _app.message("Les données ne semblent pas venir d'un fichier Ogg...", TbiApp.MESSAGE_ERREUR);
          _pret = true;
          return;
        }
      } 
      os.init(og.serialno());
      os.reset();
      vi.init();
      vc.init();
      if (os.pagein(og) < 0) {
        _app.message("Erreur de lecture de la première page d'un flux Ogg...", TbiApp.MESSAGE_ERREUR);
        _pret = true;
        return;
      }
      if (os.packetout(op) != 1) {
        // no page? must not be vorbis
        _app.message("Erreur de lecture de l'entête initial...", TbiApp.MESSAGE_ERREUR);        
        break;
      }
      if (vi.synthesis_headerin(vc, op) < 0) {
        // error case; not a vorbis header
        _app.message("Ce flux Ogg ne contient pas de données audio Vorbis...", TbiApp.MESSAGE_ERREUR);
        _pret = true;
        return;
      }
      int i = 0;
      while (i < 2) {
        while (i < 2) {
          int result = oy.pageout(og);
          if (result == 0) {
            break; // Need more data
          }
          if (result == 1) {
            os.pagein(og);
            while (i < 2) {
              result = os.packetout(op);
              if (result == 0) {
                break;
              }
              if (result == -1) {
                _app.message("Second entête corrompu...", TbiApp.MESSAGE_ERREUR);                
                break loop;
              }
              vi.synthesis_headerin(vc, op);
              i++;
            }
          }
        }
        index = oy.buffer(BUFSIZE);
        buffer = oy.data;
        try {
          bytes = _bis.read(buffer, index, BUFSIZE);
        } catch (Exception e) {
          _app.message(e, TbiApp.MESSAGE_ERREUR);
          _pret = true;
          return;
        }
        if (bytes == 0 && i < 2) {
          _app.message("Fin de fichier atteinte avant d'avoir tous les entêtes Vorbis...", TbiApp.MESSAGE_ERREUR);
          _pret = true;
          return;
        }
        oy.wrote(bytes);
      }
      byte[][] ptr = vc.user_comments;
      for (int j = 0; j < ptr.length; j++) {
        if (ptr[j] == null) {
          break;
        }
      }
      convsize = BUFSIZE / vi.channels;
      vd.synthesis_init(vi);
      vb.init(vd);
      float[][][] lPcmf = new float[1][][];
      int[] lIndex = new int[vi.channels];
       // Preset initial gain and mute
      SourceDataLine sdl = getOutputLine(vi.channels, vi.rate);
      AudioThread.setVolume(sdl, initialVolume);
      while (eos == 0 && _isPlaying) {
        while (eos == 0 && _isPlaying) {
          int result = oy.pageout(og);
          if (result == 0) {
            break; // need more data
          }
          if (result != -1) {
            os.pagein(og);
            if (og.granulepos() == 0) { 
              chained = true; 
              eos = 1;  
              break; 
            } 
            while (!Thread.currentThread().isInterrupted() && _isPlaying) {
              result = os.packetout(op);
              if (result == 0) {
                break; // need more data
              }
              if (result != -1) {
                // we have a packet. Decode it
                int samples;
                if (vb.synthesis(op) == 0) { // test for success
                  if (vd == null)
                    break;
                  vd.synthesis_blockin(vb);
                }
                samples = vd.synthesis_pcmout(lPcmf, lIndex);
                while (samples > 0 && !Thread.currentThread().isInterrupted() && _isPlaying) {
                  float[][] pcmf = lPcmf[0];
                  int bout = (samples < convsize ? samples : convsize);
                  // convert doubles to 16 bit signed ints
                  // (host order) and interleave
                  for (i = 0; i < vi.channels; i++) {
                    int iptr = i * 2;
                    int mono = lIndex[i];
                    for (int j = 0; j < bout; j++) {
                      int val = (int) (pcmf[i][mono + j] * 32767.);
                      if (val > 32767) {
                        val = 32767;
                      }
                      if (val < -32768) {
                        val = -32768;
                      }
                      if (val < 0) {
                        val = val | 0x8000;
                      }
                      convbuffer[iptr] = (byte) (val);
                      convbuffer[iptr + 1] = (byte) (val >>> 8);
                      iptr += 2 * (vi.channels);
                    }
                  }
                  outputLine.write(convbuffer, 0, 2 * vi.channels * bout);
                  if (vd != null) {
                    vd.synthesis_read(bout);
                    samples = vd.synthesis_pcmout(lPcmf, lIndex);
                  }
                }
              }
            }
            if (og.eos() != 0) {
              eos = 1;
            }
          }
        }
        if (eos == 0 && _isPlaying) {
          index = oy.buffer(BUFSIZE);
          buffer = oy.data;
          try {
            bytes = _bis.read(buffer, index, BUFSIZE);
          } catch (Exception e) {
            _app.message(e, TbiApp.MESSAGE_ERREUR);
            _pret = true;
            return;
          }
          if (bytes == -1) {
            break;
          }
          oy.wrote(bytes);
          if (bytes == 0) {
            eos = 1;
          }
        }
      }
      // Fin de diffusion du fichier ogg
      close(false);       
    }
    } else { // il y a quelque chose à jouer
        _isPlaying = true;
        _startPlaying = false;
        _pret = false;
        chained = false;
        initJOrbis();
        bytes = 0;
    }
      
    } else { // rien à jouer
        _pret = true;
    }
    } 
  }
}