/*
Copyright Caroline et Serge COUDÉ, (01/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/
package net.sergecoude.TbiGeneral;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import javax.swing.event.*;

public class PanelSelectEcran extends Panneau {

  private final static long serialVersionUID = 1L;

  private JList _listeEcrans;
  private JScrollPane _listeScroller;
  private GraphicsDevice _ecranPrincipal;
	
  public PanelSelectEcran(TbiApp app) {
    super(app);
		
    GraphicsEnvironment gEnv = GraphicsEnvironment.getLocalGraphicsEnvironment();
    GraphicsDevice[] devices = gEnv.getScreenDevices();
    _ecranPrincipal = gEnv.getDefaultScreenDevice();
		_listeEcrans = new JList(devices);
		_listeEcrans.setFont(_police);
		_listeEcrans.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    _listeEcrans.setLayoutOrientation(JList.VERTICAL);
    _listeEcrans.setCellRenderer(new ScreenRenderer(_ecranPrincipal, _police));
    _listeEcrans.setSelectedValue(_ecranPrincipal, true);
		_listeScroller = new JScrollPane(_listeEcrans);
    _listeScroller.setOpaque(false);

	  try {
      _bSuivant = new bouton(_app.getRessourceByConf("boutonsuivant"));
  		_bSuivant.addActionListener(this);
  	} catch(Exception e) {
  	 _app.log(e);
    }

    JPanel pMain = new JPanel();
    pMain.setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    pMain.setOpaque(false);
    setLayout(new BorderLayout());
    JPanel pLabel = new JPanel(new BorderLayout());
    pLabel.setOpaque(false);
    JPanel pLabel2 = new JPanel(new FlowLayout(FlowLayout.CENTER));
    pLabel2.setOpaque(false);
    pLabel.setAlignmentX(CENTER_ALIGNMENT);
    JLabel label = new JLabel("Choix de l'écran d'affichage");
    label.setFont(_police);
    pLabel2.add(label);
    pLabel.add(pLabel2, BorderLayout.SOUTH);
    _listeScroller.setAlignmentX(CENTER_ALIGNMENT);
    c.fill = GridBagConstraints.NONE;
    c.ipady = 40;      
    c.weightx = 0.0;
    c.gridwidth = 1;
    c.gridheight = 1;
    c.gridx = 0;
    c.gridy = 0;
    pMain.add(pLabel, c);
    c.gridwidth = 1;
    c.gridheight = 2;
    c.gridx = 0;
    c.gridy = 1;
    pMain.add(_listeScroller, c);
    c.gridwidth = 1;
    c.gridheight = 1;
    c.gridx = 0;
    c.gridy = 3;
    JPanel pSuivant = new JPanel(new FlowLayout(FlowLayout.CENTER));
    pSuivant.setOpaque(false);
    pSuivant.add(_bSuivant);
    pMain.add(pSuivant, c);    
    add(pMain);
  }
  
  public void actionPerformed(ActionEvent evt) {
    _app.setEcran((GraphicsDevice)_listeEcrans.getSelectedValue(), ((GraphicsDevice)_listeEcrans.getSelectedValue() == _ecranPrincipal)); 
    _app.setPanneau("Config");
  }
  
  public JList getJListEcrans() {
    return _listeEcrans;
  }

  public void init() { 
    if (_listeEcrans != null)
      if (_listeEcrans.getModel().getSize() < 2) { // on passe la sélection des écrans si qu'un seul écran
        _app.setEcran((GraphicsDevice)_listeEcrans.getModel().getElementAt(0), true); // on définit le seul écran comme écran principal
        _app.setPanneau("Config");
        return;
      }
    super.init();    
  }
  
  public void agenceLesComposants() {
    Dimension d = getSize();
    int lListe = 2*d.width/3;
    int hListe = d.height/2;
    Dimension dListe = new Dimension(lListe, hListe);
    _listeScroller.setPreferredSize(dListe);
    _listeScroller.setMinimumSize(dListe);
    _listeScroller.setMaximumSize(dListe);
    revalidate();
  }
}

class ScreenRenderer extends ResizeLabelFont implements ListCellRenderer {

  private final static long serialVersionUID = 1L;

  private GraphicsDevice _ecranPrincipal;

  public ScreenRenderer(GraphicsDevice e, Font police) {
    super("");
    _ecranPrincipal = e;
    setFont(police);
    setOpaque(true);
  }

  public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
    if (value == _ecranPrincipal)
      setText(value.toString() + " (écran principal)");
    else
      setText(value.toString()); 
    setBackground(isSelected ? Color.lightGray : Color.white);
    return this;
  }
}
